'use strict';
const { CrudService } = require('naf-framework-mongoose-free/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
const assert = require('assert');
const user = require('../model/race/user');

//
class MatchTeamGroupService extends CrudService {
  constructor(ctx) {
    super(ctx, 'matchteamgroup');
    this.model = this.ctx.model.Race.MatchTeamGroup;
    this.matchProjectModel = this.ctx.model.Race.MatchProject;
    this.teamApplyModel = this.ctx.model.Race.TeamApply;
    this.matchSignModel = this.ctx.model.Race.MatchSign;
    this.baseUserModel = this.ctx.model.Base.User;
  }
  /**
   * 根据项目和选手类型查询可选择的分组人员
   * @param {Object} query 地址栏查询条件
   * @property {String} project_id 比赛项目id
   * @property {String} person_type 选手类型
   * @property {String} team_id 小组id
   */
  async findGroupPersonSelects({ project_id, person_type, team_id }) {
    // 1.查找所有报名的选手/小队
    let allPerson;
    if (person_type === 'TeamApply') {
      allPerson = await this.teamApplyModel.find({ project_id, status: '1' });
    } else {
      allPerson = await this.matchSignModel.find({ project_id, pay_status: '1' });
    }
    // 2.查找该项目下已经分完组的成员
    const teamGroups = await this.model.find({ project_id });
    // 已经有分组的成员id集合
    let personList = _.flattenDeep(teamGroups.map(i => i.person));
    // 3.如果有team_id,则将team_id内的组员从上面有分组的成员id集合中剔除
    if (team_id) {
      const teamGroup = await this.model.findById(team_id);
      const thisTeamPerson = _.flattenDeep(teamGroup.map(i => i.person));
      personList = personList.filter(f => !thisTeamPerson.includes(f));
    }
    // 过滤掉 allPerson 中 在personList中的数据
    if (person_type === 'TeamApply') {
      allPerson = allPerson.filter(f => !personList.includes(f._id));
      return allPerson;
    }
    // 找人的信息
    allPerson = allPerson.filter(f => !personList.includes(f.user_id));
    const users = [];
    for (const i of allPerson) {
      const { user_id } = i;
      const r = await this.ctx.service.user.fetch({ id: user_id });
      users.push(r);
    }
    return users;


  }


  // 将选手的信息换出来
  async afterQuery(filter, data) {
    for (const i of data) {
      const { person_type } = i;
      if (person_type === 'User') {
        const users = await this.baseUserModel.find({ _id: i.person });
        i.person = users;
      }
    }
    return data;
  }

  async saveAll(data) {
    for (const i of data) {
      const { _id } = i;
      if (!_id) {
        // TODO: 创建前:需要检查这些人员是否出现在这个项目中的别的组中
        await this.model.create(i);
      } else {
        await this.model.updateOne({ _id }, i);
      }
    }
  }

  async auto({ match_id, group_id, project_id, team_number = 0 }) {
    // 找到比赛项目
    const project = await this.matchProjectModel.findById(project_id);
    if (!project) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '未找到比赛项目');
    const { type } = project;
    // type 为 1 双打,是TeamApply表中的, type 为2 单打,是User表
    let personList = [];
    const obj = { match_id, group_id, project_id };
    if (type === '1') {
      // 双打,找到这个项目所有的申请,且审核通过的
      obj.person_type = 'TeamApply';
      const teamApplys = await this.teamApplyModel.find({ project_id, status: '1' });
      personList = JSON.parse(JSON.stringify(teamApplys));
    } else {
      // 单打,找到这个项目所有申请报名的信息
      obj.person_type = 'User';
      const sign = await this.matchSignModel.find({ project_id, pay_status: '1' });
      personList = JSON.parse(JSON.stringify(sign));
    }
    const returnData = [];
    // 选手数检测
    const number = _.floor(_.divide(personList.length, team_number));
    if (number <= 0) throw new BusinessError(ErrorCode.DATA_INVALID, '选手不足以分组,要分组的总人数 至少为 要分的组数');
    // 选手列表分组
    const teams = _.chunk(personList, number);
    // 获取 选手被均分到每个组后的余数
    const el = personList.length % team_number;
    // 最后余下的组数据
    let last = [];
    if (el !== 0) {
      // 如果有剩余不能被均分的选手,则将这些选手 放到 last数组中,再将多出来的这个去掉
      last = _.last(teams);
      teams.pop();
    }
    // 将正常均分的小组先分出来
    for (let i = 0; i < teams.length; i++) {
      const t = { name: `小组${i + 1}`, person: teams[i], ...obj };
      returnData.push(t);
    }
    // 循环剩下的选手, 依次放入每个已经分好的组中,尽可能均分人数
    for (let i = 0; i < last.length; i++) {
      returnData[i].person.push(last[i]);
    }

    return returnData;
  }
}

module.exports = MatchTeamGroupService;
